@@ -13,21 +13,13 @@
 #include <QQmlEngine>
 #include <QQuickRenderControl>
 #include <QQuickWindow>
+#include <QScopeGuard>
 
 #include <KColorScheme>
 #include <KConfigGroup>
+#include <KIconColors>
 #include <QDBusConnection>
 
-class IconLoaderSingleton
-{
-public:
-    IconLoaderSingleton() = default;
-
-    KIconLoader self;
-};
-
-Q_GLOBAL_STATIC(IconLoaderSingleton, privateIconLoaderSelf)
-
 class StyleSingleton : public QObject
 {
     Q_OBJECT
@@ -80,8 +72,9 @@ public:
     {
         const auto key = qMakePair(cs, group);
         auto it = m_cache.constFind(key);
-        if (it != m_cache.constEnd())
+        if (it != m_cache.constEnd()) {
             return *it;
+        }
 
         using Kirigami::PlatformTheme;
 
@@ -143,14 +136,14 @@ public:
 
     void notifyWatchersPaletteChange()
     {
-        for (auto watcher : qAsConst(watchers)) {
+        for (auto watcher : std::as_const(watchers)) {
             watcher->syncColors();
         }
     }
 
     Q_SLOT void notifyWatchersConfigurationChange()
     {
-        for (auto watcher : qAsConst(watchers)) {
+        for (auto watcher : std::as_const(watchers)) {
             watcher->syncFont();
         }
     }
@@ -222,11 +215,13 @@ void PlasmaDesktopTheme::syncFont()
     KConfigGroup general(ptr->group("general"));
     setSmallFont(general.readEntry("smallestReadableFont", []() {
         auto smallFont = qApp->font();
+#ifndef Q_OS_WIN
         if (smallFont.pixelSize() != -1) {
             smallFont.setPixelSize(smallFont.pixelSize() - 2);
         } else {
             smallFont.setPointSize(smallFont.pointSize() - 2);
         }
+#endif
         return smallFont;
     }()));
 
@@ -235,16 +230,13 @@ void PlasmaDesktopTheme::syncFont()
 
 QIcon PlasmaDesktopTheme::iconFromTheme(const QString &name, const QColor &customColor)
 {
-    QPalette pal = palette();
     if (customColor != Qt::transparent) {
-        for (auto state : {QPalette::Active, QPalette::Inactive, QPalette::Disabled}) {
-            pal.setBrush(state, QPalette::WindowText, customColor);
-        }
+        KIconColors colors;
+        colors.setText(customColor);
+        return KDE::icon(name, colors);
+    } else {
+        return KDE::icon(name);
     }
-
-    privateIconLoaderSelf->self.setCustomPalette(pal);
-
-    return KDE::icon(name, &privateIconLoaderSelf->self);
 }
 
 void PlasmaDesktopTheme::syncColors()
